==> dasu_1.0.jar: edu/northwestern/dasu/util/Util.java <==
/**
 * News Project
 *
 * File:         Util.java
 * RCS:          $Id: Util.java,v 1.2 2010/05/26 19:09:35 mas939 Exp $
 * Description:  Util class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 28, 2006 at 11:21:35 AM
 * Language:     Java
 * Package:      edu.northwestern.news.util
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;



import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.LongBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.TimeZone;
import java.util.Map.Entry;



/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Util class provides utility functions for the package.
 */
public class Util {
    
	static BufferedReader in;

	private static Util self;
    
	public static long dasuTimeOffset = 0;
	
	private static boolean failed = false;
    
    public static interface PingResponse {
    	public void response(double rtt);
    }

    @SuppressWarnings("deprecation")
	public static String currentGMTDayHour() {
    	Date d = new Date(currentGMTTime());
    	String dayHour = d.getDay() + "-" + d.getHours(); 
    	return dayHour;
    }
    
    public static long currentGMTTime(){
    	return currentDasuTime();
    	//return System.currentTimeMillis()-TimeZone.getDefault().getRawOffset();
    } 
    
    public static long currentDasuTime(){
    	return System.currentTimeMillis()-dasuTimeOffset;
    }
    
    public static long fromDasuTime(long dasuTime){
    	return dasuTime+dasuTimeOffset;
    }
    
    public static void setDasuTimeOffset( long offset ){
    	
    	if (offset == -1)
    		failed = true;
    	
    	dasuTimeOffset = offset + TimeZone.getDefault().getRawOffset();
    }
    
    public static long fromGMTTime(long gmtTime){
    	return gmtTime+TimeZone.getDefault().getRawOffset();
    }
    
    public synchronized static Util getInstance(){

	    	if (self == null){
	    		self = new Util();
	    	}
    	
    	return self;
    }
    
    public static boolean isFailed() {
    	return failed;
    }

   

    /**
     * @param newEdge
     * @return
     */
    public static String getClassCSubnet(String ipaddress) {
        // TODO Auto-generated method stub
        return ipaddress.substring(0, ipaddress.lastIndexOf("."));
    }

	public static byte[] convertLong(long l) {
		byte[] bArray = new byte[8];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        LongBuffer lBuffer = bBuffer.asLongBuffer();
        lBuffer.put(0, l);
        return bArray;
	}
	
	public static  byte[] convertShort(short s) {
		byte[] bArray = new byte[2];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        ShortBuffer sBuffer = bBuffer.asShortBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertInt(int s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        IntBuffer sBuffer = bBuffer.asIntBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertFloat(float s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        FloatBuffer sBuffer = bBuffer.asFloatBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static long byteToLong(byte data[]){
		ByteBuffer bBuffer = ByteBuffer.wrap(data);
		bBuffer.order(ByteOrder.LITTLE_ENDIAN);
		LongBuffer  lBuffer = bBuffer.asLongBuffer();
		return lBuffer.get();
	}

	public static byte[] convertStringToBytes(String key) {
		try {
			return key.getBytes("ISO-8859-1");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public static String convertByteToString(byte[] value){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			baos.write(value);
		
			return baos.toString("ISO-8859-1");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static HashMap<Integer, BigDecimal> findUnionProbability(
			int numComb, HashMap<Integer, Double> perPeerProbs, 
			int maxPeersForProb) {

		HashMap<Integer,BigDecimal> indepProbs = 
			new HashMap<Integer, BigDecimal>();

		double notProbs = -1.0;
		for (Entry<Integer, Double> ent : perPeerProbs.entrySet()){
			if (notProbs<0) notProbs = 1-ent.getValue();
			else notProbs*=1-ent.getValue();
		}

		// store independent probabilities
		indepProbs.put(0, new BigDecimal(1-notProbs));

		if (perPeerProbs.size()>maxPeersForProb){
			double probs[] = new double[perPeerProbs.size()];
			int j = 0;

			for (Entry<Integer, Double> ent2 : perPeerProbs.entrySet()){
				probs[j]+= ent2.getValue();
				j++;
			}

			Arrays.sort(probs);
			double minValue = probs[perPeerProbs.size()-maxPeersForProb];
			HashMap<Integer, Double> newProbs = 
				new HashMap<Integer, Double>();
			double temp = 0;
			for (Entry<Integer,Double> ent : 
				perPeerProbs.entrySet()){


				temp= ent.getValue();

				if (temp>=minValue){
					newProbs.put(ent.getKey(), ent.getValue());
				}
			}
			// switch them over
			perPeerProbs = newProbs;
		}

		// first find all the possible terms, i.e., the unique combinations 
		// of (index+1) nodes seeing problems at the same time

		ArrayList<BigDecimal> terms = new ArrayList<BigDecimal>();
		for (Entry<Integer, Double> ent : 
			perPeerProbs.entrySet()){
			terms.add(new BigDecimal(ent.getValue(), MathContext.UNLIMITED));
		}

//		// now get the union probabilities
		BigDecimal firstParts = new BigDecimal(0);


		int n = terms.size();

		BigDecimal prob[] = new BigDecimal[n];
		for (int i = 0; i < n; i++) prob[i] = 
			terms.get(i);
		BigDecimal psum[] = new BigDecimal[n];
		BigDecimal term[] = new BigDecimal[n];
		BigDecimal prun;
		for (int j=0; j < n; j++){
			psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
			term[j] = new BigDecimal(0, MathContext.UNLIMITED);
		}
		term[n-1] = new BigDecimal(1, MathContext.UNLIMITED);
		prun = new BigDecimal(0, MathContext.UNLIMITED);

		for (int j = 0; j < n; j++){
			term[0] = term[0].add(prob[j], MathContext.UNLIMITED);
			term[n-1] = term[n-1].multiply(prob[j], MathContext.UNLIMITED);
			psum[j] = prob[j];
		}

		if (n-2>0) {

			int i2 = n-1;
			int j2 = n;
			for (int i = 1; i < i2; i++) {
				j2--;
				int k2 = j2 +1;
				for (int j = 0; j < j2; j++){
					psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
					int k1 = j+1;
					for (int k = k1; k < k2; k++){
						psum[j] = psum[j].add(prob[j].multiply(psum[k], MathContext.UNLIMITED)
								, MathContext.UNLIMITED);
					}
					term[i] = term[i].add(psum[j], MathContext.UNLIMITED);
				}
			}

		}


		for (int offset = 1; offset < numComb; offset++){
			int sign = -1;
			prun = new BigDecimal(0, MathContext.UNLIMITED);
			for (int j = offset; j < n; j++){								
				sign*=-1;
//				if (j==index2) continue;
prun=prun.add((new BigDecimal(sign)).multiply(term[j], MathContext.UNLIMITED)
		, MathContext.UNLIMITED);
			}
//			System.out.println(prun);

			indepProbs.put(offset, prun);
		}


		return indepProbs;
	}

	public static String convertIntToIp(int ip) {
		int temp;
		String out = "";
		for (int i = 0; i < 4; i++){
			temp = ip;
			temp >>= (8*(3-i));
			temp &= 0xFF;
			out+=temp+".";
		}
		return out.substring(0, out.length()-1);
	}

	public static int convertIpToInt(String ipv4) {
		String parts[] = ipv4.split("\\.");
		if (parts.length!=4) return -1;
		long ip = 0;
		long temp;
		for (int i = 0; i < parts.length; i++){
			temp = Integer.parseInt(parts[i]);
			temp <<= (8*(3-i));
			ip+=temp;
		}
		return (new Long(ip)).intValue();
	}

	public static String join(String string, Object[] array) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < array.length; i++){
			sb.append(array[i]);
			if (i<array.length-1) sb.append(string);
		}
		return sb.toString();
	}
}

==> dasu_2.02.jar: edu/northwestern/dasu/util/Util.java <==
/**
 * News Project
 *
 * File:         Util.java
 * RCS:          $Id: Util.java,v 1.4 2010/10/22 19:51:36 mas939 Exp $
 * Description:  Util class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 28, 2006 at 11:21:35 AM
 * Language:     Java
 * Package:      edu.northwestern.news.util
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;



import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.LongBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.TimeZone;
import java.util.Map.Entry;



/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Util class provides utility functions for the package.
 */
public class Util {
    
	static BufferedReader in;

	private static Util self;
    
	public static long dasuTimeOffset = 0;
	
	private static boolean failed = false;
    
    public static interface PingResponse {
    	public void response(double rtt);
    }

    @SuppressWarnings("deprecation")
	public static String currentGMTDayHour() {
    	Date d = new Date(currentGMTTime());
    	String dayHour = d.getDay() + "-" + d.getHours(); 
    	return dayHour;
    }
    
    public static long currentGMTTime(){
    	return System.currentTimeMillis()-dasuTimeOffset;
    	//return currentDasuTime();
    	//return System.currentTimeMillis()-TimeZone.getDefault().getRawOffset();
    } 
    
    public static long currentDasuTime(){
    	return currentGMTTime() + TimeZone.getDefault().getRawOffset();
    }
    
    public static long fromDasuTime(long dasuTime){
    	return dasuTime+dasuTimeOffset;
    }
    
    public static void setDasuTimeOffset( long offset ){
    	
    	if (offset == -1)
    		failed = true;
    	
    	dasuTimeOffset = offset + TimeZone.getDefault().getRawOffset();
    }
    
    public static long fromGMTTime(long gmtTime){
    	return gmtTime+TimeZone.getDefault().getRawOffset();
    }
    
    public synchronized static Util getInstance(){

	    	if (self == null){
	    		self = new Util();
	    	}
    	
    	return self;
    }
    
    public static boolean isFailed() {
    	return failed;
    }

   

    /**
     * @param newEdge
     * @return
     */
    public static String getClassCSubnet(String ipaddress) {
        // TODO Auto-generated method stub
        return ipaddress.substring(0, ipaddress.lastIndexOf("."));
    }

	public static byte[] convertLong(long l) {
		byte[] bArray = new byte[8];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        LongBuffer lBuffer = bBuffer.asLongBuffer();
        lBuffer.put(0, l);
        return bArray;
	}
	
	public static  byte[] convertShort(short s) {
		byte[] bArray = new byte[2];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        ShortBuffer sBuffer = bBuffer.asShortBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertInt(int s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        IntBuffer sBuffer = bBuffer.asIntBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertFloat(float s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        FloatBuffer sBuffer = bBuffer.asFloatBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static long byteToLong(byte data[]){
		ByteBuffer bBuffer = ByteBuffer.wrap(data);
		bBuffer.order(ByteOrder.LITTLE_ENDIAN);
		LongBuffer  lBuffer = bBuffer.asLongBuffer();
		return lBuffer.get();
	}

	public static byte[] convertStringToBytes(String key) {
		try {
			return key.getBytes("ISO-8859-1");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public static String convertByteToString(byte[] value){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			baos.write(value);
		
			return baos.toString("ISO-8859-1");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static HashMap<Integer, BigDecimal> findUnionProbability(
			int numComb, HashMap<Integer, Double> perPeerProbs, 
			int maxPeersForProb) {

		HashMap<Integer,BigDecimal> indepProbs = 
			new HashMap<Integer, BigDecimal>();

		double notProbs = -1.0;
		for (Entry<Integer, Double> ent : perPeerProbs.entrySet()){
			if (notProbs<0) notProbs = 1-ent.getValue();
			else notProbs*=1-ent.getValue();
		}

		// store independent probabilities
		indepProbs.put(0, new BigDecimal(1-notProbs));

		if (perPeerProbs.size()>maxPeersForProb){
			double probs[] = new double[perPeerProbs.size()];
			int j = 0;

			for (Entry<Integer, Double> ent2 : perPeerProbs.entrySet()){
				probs[j]+= ent2.getValue();
				j++;
			}

			Arrays.sort(probs);
			double minValue = probs[perPeerProbs.size()-maxPeersForProb];
			HashMap<Integer, Double> newProbs = 
				new HashMap<Integer, Double>();
			double temp = 0;
			for (Entry<Integer,Double> ent : 
				perPeerProbs.entrySet()){


				temp= ent.getValue();

				if (temp>=minValue){
					newProbs.put(ent.getKey(), ent.getValue());
				}
			}
			// switch them over
			perPeerProbs = newProbs;
		}

		// first find all the possible terms, i.e., the unique combinations 
		// of (index+1) nodes seeing problems at the same time

		ArrayList<BigDecimal> terms = new ArrayList<BigDecimal>();
		for (Entry<Integer, Double> ent : 
			perPeerProbs.entrySet()){
			terms.add(new BigDecimal(ent.getValue(), MathContext.UNLIMITED));
		}

//		// now get the union probabilities
		BigDecimal firstParts = new BigDecimal(0);


		int n = terms.size();

		BigDecimal prob[] = new BigDecimal[n];
		for (int i = 0; i < n; i++) prob[i] = 
			terms.get(i);
		BigDecimal psum[] = new BigDecimal[n];
		BigDecimal term[] = new BigDecimal[n];
		BigDecimal prun;
		for (int j=0; j < n; j++){
			psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
			term[j] = new BigDecimal(0, MathContext.UNLIMITED);
		}
		term[n-1] = new BigDecimal(1, MathContext.UNLIMITED);
		prun = new BigDecimal(0, MathContext.UNLIMITED);

		for (int j = 0; j < n; j++){
			term[0] = term[0].add(prob[j], MathContext.UNLIMITED);
			term[n-1] = term[n-1].multiply(prob[j], MathContext.UNLIMITED);
			psum[j] = prob[j];
		}

		if (n-2>0) {

			int i2 = n-1;
			int j2 = n;
			for (int i = 1; i < i2; i++) {
				j2--;
				int k2 = j2 +1;
				for (int j = 0; j < j2; j++){
					psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
					int k1 = j+1;
					for (int k = k1; k < k2; k++){
						psum[j] = psum[j].add(prob[j].multiply(psum[k], MathContext.UNLIMITED)
								, MathContext.UNLIMITED);
					}
					term[i] = term[i].add(psum[j], MathContext.UNLIMITED);
				}
			}

		}


		for (int offset = 1; offset < numComb; offset++){
			int sign = -1;
			prun = new BigDecimal(0, MathContext.UNLIMITED);
			for (int j = offset; j < n; j++){								
				sign*=-1;
//				if (j==index2) continue;
prun=prun.add((new BigDecimal(sign)).multiply(term[j], MathContext.UNLIMITED)
		, MathContext.UNLIMITED);
			}
//			System.out.println(prun);

			indepProbs.put(offset, prun);
		}


		return indepProbs;
	}

	public static String convertIntToIp(int ip) {
		int temp;
		String out = "";
		for (int i = 0; i < 4; i++){
			temp = ip;
			temp >>= (8*(3-i));
			temp &= 0xFF;
			out+=temp+".";
		}
		return out.substring(0, out.length()-1);
	}

	public static int convertIpToInt(String ipv4) {
		String parts[] = ipv4.split("\\.");
		if (parts.length!=4) return -1;
		long ip = 0;
		long temp;
		for (int i = 0; i < parts.length; i++){
			temp = Integer.parseInt(parts[i]);
			temp <<= (8*(3-i));
			ip+=temp;
		}
		return (new Long(ip)).intValue();
	}

	public static String join(String string, Object[] array) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < array.length; i++){
			sb.append(array[i]);
			if (i<array.length-1) sb.append(string);
		}
		return sb.toString();
	}
}
